@@ -12,13 +12,7 @@
 
 """An asynchronous SSH2 library for Python"""
 
-__author__ = 'Ron Frederick'
-
-__author_email__ = '<ronf@timeheart.net>'
-
-__url__ = 'http://asyncssh.timeheart.net/'
-
-__version__ = '0.9.0'
+from .version import __author__, __author_email__, __url__, __version__
 
 from .constants import *
 
